Crate nutex

Crate nutex 

Structs

Nutex
A nullable Mutex-like type based on tokio::sync::Mutex implementation.
NutexGuard
A custom handle that holds Nutex and contains MutexGuard of original Tokio Mutex. The guard can be held across any .await point as it is Send thanks to the original Tokio MutexGuard and Mutex.