NutexGuard

Struct NutexGuard 

pub struct NutexGuard<'a, T: Sized> { /* private fields */ }
A custom handle that holds Nutex and contains MutexGuard of original Tokio Mutex. The guard can be held across any .await point as it is Send thanks to the original Tokio MutexGuard and Mutex.

As long as you have this guard, you have exclusive access to the underlying T. The guard internally borrows the Nutex, so it guarantees that Nutex reference won’t be dropped early.

Nutex will unlock automatically when guard drop.

Implementations

Source

impl<'a, T: Sized> NutexGuard<'a, T>

pub fn nutex(&self) -> &'a Nutex<T>

Returns a reference to the inner Nutex that lives the same time as the guard.